/*
 * 작성일: 2005. 7. 7.
 * 작성자: 이강우
 * 설  명: 
 */
package net.koov.irc;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Vector;

/**
 * @작성자 이강우
 * @작성일 2005. 7. 7.
 * @수정일 2005. 7. 7.
 *
 */
public class KoovBot {
	
	// IO 관련 변수들
	private Socket					socket;
	private BufferedReader			in;
	private BufferedWriter			out;
	
	private static PrintWriter		log;
	
	// 서버 관련 변수들
	private static String			serverVersion = "0.0.1";
	private boolean					running;
	
	private boolean					connected = false;
	
	private long					lastSendTime = 0;
	private int MAX_PERMIT_LINES	= 5; // 최대 출력 라인
	
	// 봇 관련변수들

	/**
	 * 
	 */
	public KoovBot() {
		init();
	}

	/**
	 * 초기화 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 11.
	 * @수정일: 2005. 7. 11.
	 *
	 */
	private void init() {
		try {
			if (log != null) {
				log.flush();
				log.close();
				log = null;
			}
			
			// Append모드
			log = new PrintWriter(new FileWriter("log.txt", true));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * 서버에 접속하는 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 *
	 */
	protected void connect() {
		try {
			socket = new Socket(Config.getServerAddress(), Config.getServerPort());
			System.out.println("Connecting to server....");
		} catch (Exception e) {
			System.err.println("Couldn't make connection .. try again!!!");
			e.getMessage();
			e.printStackTrace();
			System.err.println(Config.getServerAddress() + " " + Config.getServerPort());
			System.exit(0);
		}

		try {
			in		= new BufferedReader(new InputStreamReader(socket.getInputStream()));
			out		= new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
		} catch (Exception e) {
			System.err.println("Couldn't get I/O-Streams .. Exiting!");
			e.printStackTrace();
			System.exit(0);
		}

		return;
	}

	/**
	 * 접속끊기
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 *
	 */
	protected void disconnect() {
		try //Streams beenden
		{
			in.close();
			out.close();
		}

		catch (IOException e) {
			System.err.print("Error occured while trying to disconnect from IRC server");
			e.printStackTrace();
			System.exit(0);
		}

	}

	/**
	 * IRC 서버에 로그인 하는 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 *
	 */
	protected void logon() {
		try //sending user info
		{
			System.out.println("logon");
			
			// Command:  USER
			// Parameters:  <username> <hostname> <servername> <realname> 
			out.write("user " + Config.getClientName().toLowerCase() + " " + Config.getClientAddress().toLowerCase() + " " + Config.getServerAddress().toLowerCase() + ": " + Config.getClientName());
			out.newLine();
			out.write("nick " + Config.getClientNick());
			out.newLine();
			
			System.out.println("Connected, logging on to IRC server " + Config.getServerAddress() + "!");
			
			//sendToServer("PRIVMSG nickserv :IDENTIFY " + botName.toLowerCase());
			out.newLine();
			out.flush();
			
		} catch (Exception e) {
			System.err.println("Error occured while logging on to IRC server ... Exiting!");
			e.printStackTrace();
			System.exit(0);
		}
	}

	/**
	 * IRC 서버에서 로그오프하는 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 *
	 */
	protected void logoff() {
		try {
			//if (!sendToServer("logging off from server");
			
			out.write("logging off from server");
			out.newLine();
			out.flush();
			
		} catch (Exception e) {
			System.err.println("Error occured while logging off from server");
			e.printStackTrace();
			System.exit(0);
		}

	}

	/**
	 * 서버로 메세지를 날리는 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 * @param message
	 * @return
	 */
	protected boolean send(String message) {
//		debug("SEND:" + message);
		
		System.out.println("SEND:" + message);
		
		if (message == null || message.trim().length() < 1) {
			return true;
		}
		
		try {
			
			if (System.currentTimeMillis() - this.lastSendTime < 500) {
				Thread.sleep(500 - (System.currentTimeMillis() - this.lastSendTime));
			}
			
			out.write(message);
			out.newLine();
			out.flush();
			
			this.lastSendTime = System.currentTimeMillis();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return true;
	}

	/**
	 * 메인 서비스 메소드
	 * 데이타를 읽어서 프로세스를 처리한다.
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 *
	 */
	protected void service() //serves and handles IRC msgs
	{
		try {
			
			if (in.ready()) {
				String message = in.readLine();
				
//				debug(message);
				
				if (message != null && message.length() > 0) {
					if (message.charAt(0) == ':') {
						processCommand(message.substring(1));
					}
					else {
						processFunction(message);
					}
				}
			}
		} catch (Exception e) {
			System.err.println("Error occured while handling messages");
			e.printStackTrace();
		}
	}

	/**
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 11.
	 * @수정일: 2005. 7. 11.
	 * @param message
	 */
	private void processFunction(String message) {
		String[] tokens = message.split(" ");
		
		if (tokens != null && tokens.length > 0) {
			// PING 메세지
			if (tokens[0].equalsIgnoreCase("PING")) {
				send("PONG " + tokens[1]);
			}
			else {
//				debug("Unknown Function : " + message);
			}
		}
	}

	/**
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 11.
	 * @수정일: 2005. 7. 11.
	 * @param string
	 * 
	 * 모든 커맨드를 분기 처리하여 주는 메소드
	 */
	private void processCommand(String string) {
		String[] tokens = string.split(":", 2);
		String[] headers = tokens[0].split(" ");
		
		String command = null;
		String[] userInfo = null;
		
		if (headers.length > 2) {
			command = headers[1];
			userInfo = getUserInfoFromHeader(headers[0]);
			
			// 공지사항
			if (command.equalsIgnoreCase(IRC.COMMAND.NOTICE)) {
				connected = true;
			}
			// 귓말일경우
			else if (command.equalsIgnoreCase(IRC.COMMAND.PRIVMSG)) {
				processOrder(headers, tokens[1]);
			}
			
		}
		
		System.out.println("-----------------------------------------------");
		System.out.println(string);
		for (int i = 0 ; i < headers.length ; i++) {
			System.out.println("HEADER[" + i + "] "+ headers[i]);
		}
		if (tokens.length > 1) {
			System.out.println("CONTENTS " + tokens[1]);
		}
		System.out.println("\r\n\r\n");
	}
	
	/**
	 * 사용자 정보를 가져온다.
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 20.
	 * @수정일: 2005. 7. 20.
	 * @param str
	 * @return
	 */
	private String[] getUserInfoFromHeader(String str) {
		String[] info = null;
		
		if (str != null && str.indexOf("!") != -1 && str.indexOf("@") != -1) {
			info = new String[3];
			info[0] = str.substring(0, str.indexOf("!"));
			info[1] = str.substring(str.indexOf("!") + 1, str.indexOf("@"));
			info[2] = str.substring(str.indexOf("@") + 1);
		}
		
		return info;
	}
	
	/**
	 * 명령을 처리하는 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 11.
	 * @수정일: 2005. 7. 11.
	 * @param who
	 * @param order
	 */
	private void processOrder(String[] headers, String msg) {
		String who = headers[0];
		String id = null;
		String hostUser = null;
		String hostAt = null;
		String channel = null;
		
		if (headers.length >= 3 && headers[2] != null && headers[2].charAt(0) == '#') {
			channel = headers[2];
		}
		
		if (who != null && who.indexOf("!") != -1 && who.indexOf("@") != -1) {
			id = who.substring(0, who.indexOf("!"));
			hostUser = who.substring(who.indexOf("!") + 1, who.indexOf("@"));
			hostAt = who.substring(who.indexOf("@") + 1);
		}
		
		if (msg.length() > 0 && msg.charAt(0) == '!') {
			String[] orders = msg.split(" ");
			
			// 조인 명령어
			if (orders[0].equalsIgnoreCase("!join") && orders.length >= 2) {
				if (isAuthUser(id, hostUser, hostAt)) {
					String tmpChan = "";
					for (int x = 1 ; x < orders.length ; x++) {
						tmpChan += orders[x] + ",";
					}
					
					send("JOIN " + tmpChan);
				}
				else {
					send("PRIVMSG " + id + " :권한이 없네요~");
				}
			}
			else if (orders[0].equalsIgnoreCase("!run") || orders[0].equalsIgnoreCase("!실행")) {
				// 권한검사
				if (!isAuthUser(id, hostUser, hostAt)) {
					send("PRIVMSG " + id + " :권한이 없네요");
				}
				else if (orders.length > 1){
					
					Object[] result = getRuntimeResult(msg.substring(msg.indexOf(' ') + 1));
					
					if (channel != null && result.length > 0) {
						for (int i = 0 ; i < result.length ; i++) {
							send("PRIVMSG " + channel + " :" + result[i].toString());
							
							if (i > MAX_PERMIT_LINES) {
								send("PRIVMSG " + channel + " :>> 결과가 너무 길어 중단합니다.");
								break;
							}
						}
					}
					
				}
				else {
					send("PRIVMSG " + id + " :실행할 명령을 입력해주세요");
				}
			}
			else if (orders[0].equalsIgnoreCase("!part") || orders[0].equalsIgnoreCase("!나가")) {
				// 권한검사
				if (!isAuthUser(id, hostUser, hostAt)) {
					send("PRIVMSG " + id + " :권한이 없네요");
				}
				else if (channel == null) {
					if (orders.length < 2) {
						send("PRIVMSG " + id + " :채널을 지정해주세요~");
					}
					else {
						if (isAuthUser(id, hostUser, hostAt)) {
							String tmpChans = "";
							for (int x = 1 ; x < orders.length ; x++) {
								tmpChans += orders[x] + ",";
							}
							
							send("PART " + tmpChans);
						}
					}
				}
			}
			// 옵줘
			else if (orders[0].equalsIgnoreCase("!op") || orders[0].equalsIgnoreCase("!옵")) {
				// 권한검사
				if (!isAuthUser(id, hostUser, hostAt)) {
					send("PRIVMSG " + id + " :권한이 없네요~");
				}
				else if (channel == null) {
					send("PRIVMSG " + id + " :채널을 지정해주세요~");
				}
				// 그냥 !op이라고만 쳤을경우 자기한테 옵주기
				else if (orders.length < 2) {
					send("MODE " + channel + " +o " + id);
				}
				else {
					if (isAuthUser(id, hostUser, hostAt)) {
						String tmpUsers = "";
						String tmpOps = "";
						for (int x = 1 ; x < orders.length ; x++) {
							tmpUsers += orders[x] + " ";
							tmpOps += "o";
						}
						
						send("MODE " + channel + " +" + tmpOps + " " + tmpUsers);
					}
				}
			}
			else if (orders[0].equalsIgnoreCase("!allop") || orders[0].equalsIgnoreCase("!!")) {
				
			}
		}
		
		System.out.println("id:" + id + "/hostUser:" + hostUser + "/hostAt:" + hostAt);
	}

	/**
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 20.
	 * @수정일: 2005. 7. 20.
	 * @param string
	 * @return
	 */
	private Object[] getRuntimeResult(String string) {
		Vector result = new Vector();
		Runtime rt = Runtime.getRuntime();
		Process pr = null;
		
		BufferedReader br = null;
		
		try {
			System.out.println("명령 시작");
			pr = rt.exec(string);
			br = new BufferedReader(new InputStreamReader(pr.getInputStream()));
			
			String line = null;
			for (int x = 0 ; x < (MAX_PERMIT_LINES * 5) && (line = br.readLine()) != null ; x++) {
				if (line.trim().length() < 1) {
					continue;
				}
				
				result.add(line);
			}
			
			System.out.println("명령 종료");
		} catch (IOException e) {
			result.add("명령 수행중 에러가 발생하였습니다.");
		} finally {
			if (pr != null) {
				pr.destroy();
			}
		}
		
		return result.toArray();
	}

	/**
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 11.
	 * @수정일: 2005. 7. 11.
	 * @param id
	 * @param hostUser
	 * @param hostAt
	 * @return
	 */
	private boolean isAuthUser(String id, String hostUser, String hostAt) {
		boolean result = false;
		
		Vector authUser = Config.getAccounts();
		
		for (int i = 0 ; i < authUser.size() ; i++) {
			String[] user = (String[]) authUser.get(i);
			if (id.matches(user[0]) && hostAt.matches(user[2])) {
				result = true;
			}
		}
		
		return result;
	}

	/**
	 * 메인 스타트 메소드
	 * @작성자: 이강우
	 * @작성일: 2005. 7. 7.
	 * @수정일: 2005. 7. 7.
	 * @param args
	 */
	public static void main(String[] args) {
		System.out.println("+==============================================+\r\n");
		System.out.println("  KooV Bot " + serverVersion + " - Developed by KooV");
		System.out.println("+==============================================+\r\n");
		System.out.println("\n\n");

		KoovBot kb = new KoovBot();
		kb.connect();
		kb.logon();
		kb.running = true;

		while (kb.running) {
			kb.service();
			
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		kb.logoff();
		kb.disconnect();
	}
	
	public void debug(String msg) {
		if (log != null) {
			log.println(msg);
			log.flush();
		}
		
		System.out.println(msg);
	}
}
